/*
  DMA_test
  Programa que utiliza el controlador de DMA para realizar transferencias de datos.
  Se utiliza una lista enlazadas de descriptores para indicar las transferencias.
  Se usan interrupciones para avisar del final de la transferencia.
  Desarrollado en C utilizando macros del CMSIS.
*/


// Declaración de constantes
// -------------------------
#define SIZE 2048                     // Tamaño del bloque a transferir
#define PULSADOR  13                  // Pin al que está conectado el pulsador
#define DMAC_MEMCH  5                 // Declaración de DMAC_MEMCH para utilizar el canal 5 del DMA
#define DMAC_CBTC  DMAC_EBCIER_CBTC5  // Declaración de DMAC_CBTC para utilizar el canal 5 del DMA
#define MAXSIZE 2048                  // Tamaño máximo de una transferencia con DMA


// Declaración de variables globales
// ---------------------------------
// Lista enlazada de descriptores, uno por bloque a copiar (solo uno en este ejemplo)
struct DMAllist {
  uint32_t srcAdd, destAdd, ctrla, ctrlb, next;  
} desc[1];
// ---
uint32_t  dataBuffer[2 * MAXSIZE];    // Zona de memoria en la que se realizarn las copias
// ---
static volatile uint32_t DMADone=0;   // Se pone a 1 por la rutina de tratamiento del DMA
static volatile uint32_t status;      // Se modifica con el estado del DMA por la rutina de tratamiento del DMA



// Rutina de tratamiento de las interrupciones generadas por el DMA
// ----------------------------------------------------------------
void DMAC_Handler() {
  //Serial.print(" -(tratando la interrupción)- ");
  status = DMAC->DMAC_EBCISR; // Lee el registro de estado para poner a cero el indicador de que ha terminado
  DMADone = 1;                // Modifica la variable global DMADone para indicar que el DMA ha acabado
}


// Setup
// -----
void setup() {
  uint32_t  stat;
  
  Serial.begin(9600);               // Habilita la comunicación por el puerto serie
  pinMode(PULSADOR, INPUT_PULLUP);  // Configura el pin del pulsador como entrada con pull up

  Serial.println("\n");
  Serial.println("CONFIGURACIÓN INICIAL");
  Serial.println("---------------------");
  
  Serial.println("Rellenamos la lista de descriptores (solo hay uno) para las copias...");
  desc[0].srcAdd  = (uint32_t)&dataBuffer[0];
  desc[0].destAdd = (uint32_t)&dataBuffer[MAXSIZE];
  desc[0].ctrla   = SIZE | DMAC_CTRLA_SRC_WIDTH_WORD | DMAC_CTRLA_DST_WIDTH_WORD;
  desc[0].ctrlb   = DMAC_CTRLB_SRC_DSCR_FETCH_FROM_MEM | DMAC_CTRLB_DST_DSCR_FETCH_FROM_MEM |
                    DMAC_CTRLB_FC_MEM2MEM_DMA_FC | DMAC_CTRLB_SRC_INCR_INCREMENTING |
                    DMAC_CTRLB_DST_INCR_INCREMENTING;
  desc[0].next    = 0; // no hay más elementos en la lista

  Serial.print("Inicio de la memoria reservada ...... 0x");
  Serial.println((uint32_t)&dataBuffer, HEX);
  Serial.print("Inicio de la lista de descriptores .. 0x");
  Serial.println((uint32_t)&desc, HEX);
  Serial.print("Descriptor 0 ->  dir. origen: 0x");
  Serial.print(desc[0].srcAdd, HEX);
  Serial.print(", dir. destino: 0x");
  Serial.print(desc[0].destAdd, HEX);
  Serial.print(", CTRLA: 0x");
  Serial.print(desc[0].ctrla, HEX);
  Serial.print(", CTRLB: 0x");
  Serial.print(desc[0].ctrlb, HEX);
  Serial.print("   Siguiente: ");
  Serial.print(desc[0].next, HEX);
  Serial.println("");
  
  Serial.print("Programando la copia con el controlador de DMA...");
  pmc_enable_periph_clk(ID_DMAC);
  dmac_disable();
  DMAC->DMAC_GCFG = DMAC_GCFG_ARB_CFG_FIXED;
  NVIC_DisableIRQ(DMAC_IRQn);
  NVIC_SetPriority(DMAC_IRQn, 0);
  NVIC_EnableIRQ(DMAC_IRQn);
  stat = DMAC->DMAC_EBCISR;       // clear interrupts
  DMAC->DMAC_EBCIER |= DMAC_CBTC; // enable interrupt
  dmac_enable();
  dmac_channel_disable(DMAC_MEMCH);
  DMAC->DMAC_CH_NUM[DMAC_MEMCH].DMAC_CFG = DMAC_CFG_SOD | DMAC_CFG_FIFOCFG_ASAP_CFG;
  DMAC->DMAC_CH_NUM[DMAC_MEMCH].DMAC_DSCR = (uint32_t)desc;
  Serial.println(" programada!");
  Serial.println("");

}

// Loop
// ----
void loop(){  
  register uint32_t cnt = 0;   // Para incrementarlo mientras el DMA hace las transferencias
  unsigned long  t;            // Para medir el tiempo de la transferencia
  uint32_t  stat;

  Serial.println("");
  Serial.println("TRANSFERENCIA POR DMA");
  Serial.println("---------------------");
  Serial.println("");

  Serial.print("Se va a copiar por DMA un bloque de memoria de ");
  Serial.print(SIZE);
  Serial.println(" palabras.");
  Serial.println("");


  Serial.println("Pulsa para empezar la transferencia.\n");
  while (digitalRead(PULSADOR) == HIGH);

  Serial.print("Comenzamos la transferencia...");
  dmac_channel_enable(DMAC_MEMCH);
  t = micros();

  // Mientras se realiza la transferencia, el microprocesador puede estar haciendo
  //   otras cosas, por ejemplo, incrementando el contador cnt
  while(!DMADone) {
    cnt++;
  }
  t = micros() - t;
  
  // Puesta a 0 del DMA
  DMADone = 0;
  dmac_channel_disable(DMAC_MEMCH);
  
  // Mostramos los resultados
  Serial.println(" transferencia terminada!\n");
  Serial.print("Valor del contador cnt ......... ");
  Serial.println(cnt);
  Serial.print("Tiempo de la transferencia ..... ");
  Serial.print(t);
  Serial.println(" us");
  Serial.println("");  

  while(1);            // Bucle infinito
}


// Funciones para la gestión del DMA
// ---------------------------------
// Habilita el controlador de DMA
static void dmac_enable() {
  DMAC->DMAC_EN = DMAC_EN_ENABLE;
}
// Deshabilita el controlador de DMA
static void dmac_disable() {
  DMAC->DMAC_EN &= (~DMAC_EN_ENABLE);
}
// Habilita el canal de DMA
static void dmac_channel_enable(uint32_t ul_num) {
  DMAC->DMAC_CHER = DMAC_CHER_ENA0 << ul_num;
}
// Deshabilita el canal de DMA
static void dmac_channel_disable(uint32_t ul_num) {
  DMAC->DMAC_CHDR = DMAC_CHDR_DIS0 << ul_num;
}
// Consulta si la transferencia se ha completado
static bool dmac_channel_transfer_done(uint32_t ul_num) {
  return (DMAC->DMAC_CHSR & (DMAC_CHSR_ENA0 << ul_num)) ? false : true;
}

